// src/globals.ts
var _URL = URL;
var _URLSearchParams = URLSearchParams;
var _Headers = Headers;

// src/error/JsonParseError.ts
var JsonParseError = class extends Error {
  constructor(message, jsonString) {
    super(`${message}
${jsonString}`);
  }
};

// src/parse.ts
import * as v from "valibot";

// src/object.ts
function isObject(item) {
  return item != null && typeof item === "object" && !Array.isArray(item);
}
function mergeDeep(target, ...sources) {
  if (!sources.length) return target;
  const source = sources.shift();
  if (isObject(target) && isObject(source)) {
    for (const key in source) {
      if (isObject(source[key])) {
        if (!target[key]) Object.assign(target, { [key]: {} });
        mergeDeep(target[key], source[key]);
      } else {
        Object.assign(target, { [key]: source[key] });
      }
    }
  }
  return mergeDeep(target, ...sources);
}

// src/parse.ts
function stringToJsonWithErrorHandling(string2, errorMessage) {
  try {
    return JSON.parse(string2);
  } catch (error) {
    throw new JsonParseError(errorMessage ?? "Unable to parse string to JSON.", string2);
  }
}
function parseWithErrorHandling(schema, data, customErrorMessage) {
  const parseResult = v.safeParse(schema, data);
  if (!parseResult.success) {
    throw new ValidationError(
      customErrorMessage ?? `Error validating schema with data ${JSON.stringify(data)}`,
      parseResult.issues
    );
  }
  return parseResult.output;
}
function valibotRecursiveFlattenIssues(issues) {
  let flattened = v.flatten(issues);
  for (const issue of issues) {
    if (issue.issues) {
      flattened = mergeDeep(flattened, valibotRecursiveFlattenIssues(issue.issues));
    }
  }
  return flattened;
}

// src/error/ValidationError.ts
var ValidationError = class extends Error {
  constructor(message, valibotIssues = []) {
    const errorDetails = valibotIssues.length > 0 ? JSON.stringify(valibotRecursiveFlattenIssues(valibotIssues), null, 2) : "No details provided";
    super(`${message}
${errorDetails}`);
    this.valibotIssues = valibotIssues;
  }
};

// src/error/InvalidFetchResponseError.ts
var InvalidFetchResponseError = class extends Error {
  constructor(message, textResponse, response) {
    super(`${message}
${textResponse}`);
    this.textResponse = textResponse;
    this.response = response.clone();
  }
};

// src/date.ts
function dateToSeconds(date) {
  const milliseconds = date?.getTime() ?? Date.now();
  return Math.floor(milliseconds / 1e3);
}
function addSecondsToDate(date, seconds) {
  return new Date(date.getTime() + seconds * 1e3);
}

// src/encoding.ts
import { Buffer } from "buffer";
function decodeUtf8String(string2) {
  return new Uint8Array(Buffer.from(string2, "utf-8"));
}
function encodeToUtf8String(data) {
  return Buffer.from(data).toString("utf-8");
}
function decodeBase64(base64) {
  return new Uint8Array(Buffer.from(base64, "base64"));
}
function encodeToBase64(data) {
  if (typeof data === "string") {
    return Buffer.from(data).toString("base64");
  }
  return Buffer.from(data).toString("base64");
}
function encodeToBase64Url(data) {
  return base64ToBase64Url(encodeToBase64(data));
}
function base64ToBase64Url(base64) {
  return base64.replace(/\+/g, "-").replace(/\//g, "_").replace(/=/g, "");
}

// src/path.ts
function joinUriParts(base, parts) {
  if (parts.length === 0) return base;
  let combined = base.trim();
  combined = base.endsWith("/") ? base.slice(0, base.length - 1) : base;
  for (const part of parts) {
    let strippedPart = part.trim();
    strippedPart = strippedPart.startsWith("/") ? strippedPart.slice(1) : strippedPart;
    strippedPart = strippedPart.endsWith("/") ? strippedPart.slice(0, strippedPart.length - 1) : strippedPart;
    if (strippedPart === "") continue;
    combined += `/${strippedPart}`;
  }
  return combined;
}

// src/url.ts
function getQueryParams(url2) {
  const parsedUrl = new _URL(url2);
  const searchParams = new _URLSearchParams(parsedUrl.search);
  const params = {};
  searchParams.forEach((value, key) => {
    params[key] = value;
  });
  return params;
}
function objectToQueryParams(object) {
  const params = new _URLSearchParams();
  for (const [key, value] of Object.entries(object)) {
    if (value != null) {
      params.append(key, typeof value === "object" ? JSON.stringify(value) : String(value));
    }
  }
  return params;
}

// src/valibot-fetcher.ts
import * as v2 from "valibot";

// src/content-type.ts
var ContentType = /* @__PURE__ */ ((ContentType2) => {
  ContentType2["XWwwFormUrlencoded"] = "application/x-www-form-urlencoded";
  ContentType2["Json"] = "application/json";
  ContentType2["JwkSet"] = "application/jwk-set+json";
  return ContentType2;
})(ContentType || {});
function isContentType(contentType, value) {
  return value.toLowerCase().trim().split(";")[0] === contentType;
}
function isResponseContentType(contentType, response) {
  const header = response.headers.get("Content-Type");
  if (!header) return false;
  return isContentType(contentType, header);
}

// src/valibot-fetcher.ts
var defaultFetcher = fetch;
function createValibotFetcher(fetcher = defaultFetcher) {
  return async (schema, expectedContentType, ...args) => {
    const response = await fetcher(...args);
    if (response.ok && !isResponseContentType(expectedContentType, response)) {
      throw new InvalidFetchResponseError(
        `Expected response to match content type '${expectedContentType}', but received '${response.headers.get("Content-Type")}'`,
        await response.clone().text(),
        response
      );
    }
    return {
      response,
      result: response.ok ? v2.safeParse(schema, await response.json()) : void 0
    };
  };
}

// src/validation.ts
import * as v3 from "valibot";

// src/config.ts
var GLOBAL_CONFIG = {
  allowInsecureUrls: false
};
function setGlobalConfig(config) {
  GLOBAL_CONFIG = config;
}
function getGlobalConfig() {
  return GLOBAL_CONFIG;
}

// src/validation.ts
var vHttpsUrl = v3.pipe(
  v3.string(),
  v3.url(),
  v3.check((url2) => {
    const { allowInsecureUrls } = getGlobalConfig();
    return allowInsecureUrls ? url2.startsWith("http://") || url2.startsWith("https://") : url2.startsWith("https://");
  }, "url must be an https:// url")
);
var vInteger = v3.pipe(v3.number(), v3.integer());
var vHttpMethod = v3.picklist(["GET", "POST", "PUT", "DELETE", "HEAD", "OPTIONS", "TRACE", "CONNECT", "PATCH"]);

// src/array.ts
function arrayEqualsIgnoreOrder(a, b) {
  if (new Set(a).size !== new Set(b).size) return false;
  if (a.length !== b.length) return false;
  return a.every((k) => b.includes(k));
}

// src/www-authenticate.ts
var unquote = (value) => value.substring(1, value.length - 1).replace(/\\"/g, '"');
var sanitize = (value) => value.charAt(0) === '"' ? unquote(value) : value.trim();
var body = (
  // biome-ignore lint/suspicious/noControlCharactersInRegex: <explanation>
  /((?:[a-zA-Z0-9._~+\/-]+=*(?:\s+|$))|[^\u0000-\u001F\u007F()<>@,;:\\"/?={}\[\]\u0020\u0009]+)(?:=([^\\"=\s,]+|"(?:[^"\\]|\\.)*"))?/g
);
var parsePayload = (scheme, string2) => {
  const payload = {};
  while (true) {
    const res = body.exec(string2);
    if (!res) break;
    const [, key, newValue] = res;
    const payloadValue = payload[key];
    if (newValue) {
      const sanitizedValue = sanitize(newValue);
      payload[key] = payloadValue ? Array.isArray(payloadValue) ? [...payloadValue, sanitizedValue] : [payloadValue, sanitizedValue] : sanitizedValue;
    } else if (!payloadValue) {
      payload[key] = null;
    }
  }
  return { scheme, payload };
};
function parseWwwAuthenticateHeader(str) {
  const start = str.indexOf(" ");
  let scheme = str.substring(0, start);
  let value = str.substring(start);
  const challenges = [];
  const endsWithSchemeRegex = /, ?(Bearer|DPoP|Basic)$/;
  const endsWithSchemeTest = endsWithSchemeRegex.exec(value);
  let endsWithScheme = void 0;
  if (endsWithSchemeTest) {
    value = value.substring(0, value.length - endsWithSchemeTest[0].length);
    endsWithScheme = endsWithSchemeTest[1];
  }
  const additionalSchemesRegex = /(.*?)(, ?)(Bearer|DPoP|Basic)[, ]/;
  let match = additionalSchemesRegex.exec(value);
  while (match) {
    challenges.push(parsePayload(scheme, match[1]));
    value = value.substring(match[0].length - 1);
    scheme = match[3];
    match = additionalSchemesRegex.exec(value);
  }
  challenges.push(parsePayload(scheme, value));
  if (endsWithScheme) {
    challenges.push({ scheme: endsWithScheme, payload: {} });
  }
  return challenges;
}
function encodeWwwAuthenticateHeader(challenges) {
  const entries = [];
  for (const challenge of challenges) {
    const encodedParams = Object.entries(challenge.payload).flatMap(([key, value]) => {
      const encode = (s) => s.replace(/\\/g, "\\\\").replace(/"/g, '\\"');
      if (Array.isArray(value)) {
        return value.map((v4) => `${key}="${encode(v4)}"`);
      }
      return value ? `${key}="${encode(value)}"` : key;
    });
    entries.push(encodedParams.length === 0 ? challenge.scheme : `${challenge.scheme} ${encodedParams.join(", ")}`);
  }
  return entries.join(", ");
}
export {
  ContentType,
  _Headers as Headers,
  InvalidFetchResponseError,
  JsonParseError,
  _URL as URL,
  _URLSearchParams as URLSearchParams,
  ValidationError,
  addSecondsToDate,
  arrayEqualsIgnoreOrder,
  createValibotFetcher,
  dateToSeconds,
  decodeBase64,
  decodeUtf8String,
  defaultFetcher,
  encodeToBase64,
  encodeToBase64Url,
  encodeToUtf8String,
  encodeWwwAuthenticateHeader,
  getGlobalConfig,
  getQueryParams,
  isContentType,
  isResponseContentType,
  joinUriParts,
  mergeDeep,
  objectToQueryParams,
  parseWithErrorHandling,
  parseWwwAuthenticateHeader,
  setGlobalConfig,
  stringToJsonWithErrorHandling,
  vHttpMethod,
  vHttpsUrl,
  vInteger,
  valibotRecursiveFlattenIssues
};
//# sourceMappingURL=index.mjs.map